#include "mastertab.h"

#include <dirent.h> //Lecture du contenu du dossier de roms
#include <iostream> //cout
#include <fstream> //Lecture du fichier de config
#include <QString>

#include "masterthread.h"
#include "consoletab.h"

using namespace std;

void init_probas(QDoubleSpinBox *dsb){
    dsb->setMaximum(1.0);
    dsb->setMinimum(0.0);
    dsb->setSingleStep(0.05);
}

void select_choice(QComboBox *qcb, QString value){
    int i;
    for(i=0;i<qcb->count(); i++){
        if(qcb->itemText(i) == value) qcb->setCurrentIndex(i);
    }
}


MasterTab::MasterTab(MainWindow *window){

    this->window = window;
    layout = new QHBoxLayout;

    rom_path = new LabelledWidget<QComboBox>("Rom");
    
    DIR *rom_dir = opendir("../ale/roms");
    if(rom_dir == NULL){
        cout << "Could not open ALE rom directory\n";
    }else{
        struct dirent *ent;
        while((ent = readdir(rom_dir))!= NULL){
            if(ent->d_name[0]=='.') continue;
            rom_path->getChild()->addItem(ent->d_name);
        }
    }

    colour_palette = new LabelledWidget<QComboBox>("Color Palette");
    colour_palette->getChild()->addItems({"SECAM", "PAL", "standard"});

    display_screen = new LabelledWidget<QCheckBox>("Display Screen");
    enable_sound = new LabelledWidget<QCheckBox>("Enable Sound");
    fresh_pop = new LabelledWidget<QCheckBox>("Fresh Pop");

    repeat_action_prob = new LabelledWidget<QDoubleSpinBox>("Repeat action proba");
    mut_rate = new LabelledWidget<QDoubleSpinBox>("Mutation rate");
    cross_rate = new LabelledWidget<QDoubleSpinBox>("Crossover rate");

    n_eval_p = new LabelledWidget<QSpinBox>("Nb tests / agent");

    n_eval_s = new LabelledWidget<QSpinBox>("Nb tests max by a slave on an agent");
    tourn_size = new LabelledWidget<QSpinBox>("Tournament Size");
    pop_size = new LabelledWidget<QSpinBox>("Population size");
    nb_iter = new LabelledWidget<QSpinBox>("Nb Iterations");

    init_probas(repeat_action_prob->getChild());
    init_probas(mut_rate->getChild());
    init_probas(cross_rate->getChild());

    this->loadCfg();

    QVBoxLayout *left = new QVBoxLayout;
    QVBoxLayout *right = new QVBoxLayout;

    left->addWidget(rom_path->getWidget());
    left->addWidget(display_screen->getWidget());
    left->addWidget(enable_sound->getWidget());
    left->addWidget(repeat_action_prob->getWidget());
    left->addWidget(colour_palette->getWidget());
    left->addWidget(fresh_pop->getWidget());
    left->addWidget(n_eval_p->getWidget());
    right->addWidget(n_eval_s->getWidget());
    right->addWidget(mut_rate->getWidget());
    right->addWidget(cross_rate->getWidget());
    right->addWidget(tourn_size->getWidget());
    right->addWidget(pop_size->getWidget());
    right->addWidget(nb_iter->getWidget());

    

    start = new QPushButton("Start");
    QObject::connect(start,SIGNAL(clicked()),this,SLOT(launchMaster()));
    right->addWidget(start);

    layout->addLayout(left);
    layout->addLayout(right);

    this->setLayout(layout);


}


void MasterTab::loadCfg(){

    ifstream config;
    config.open("../src/config.config");

    if(config.is_open()){

        string line;
        while (getline(config,line)){

            if(line.length() < 2) continue;
            if(line.at(0) == '/' && line.at(1) == '/') continue;
            
            size_t pos = line.find("=");
            string param = line.substr(0,pos);
            string value = line.substr(pos+1);

            if(param == "romPath"){
                select_choice(rom_path->getChild(),
                    QString::fromStdString(
                        value.substr(value.find_last_of("/") + 1)
                    ));
	    	}else if(param == "displayScreen"){
                if(value == "true") display_screen->getChild()->setChecked(true);
    		}else if(param == "enableSound"){
    			if(value == "true") enable_sound->getChild()->setChecked(true);
    		}else if(param == "repeatActionProb"){
                repeat_action_prob->getChild()->setValue(
                    stod(value)
                );
    		}else if(param == "colourPalette"){
    			select_choice(
                    colour_palette->getChild(),
                    QString::fromStdString(value)
                );
            }else if(param == "freshPop"){
                if(value == "true") fresh_pop->getChild()->setChecked(true);
            }else if(param == "nEvalP"){
                n_eval_p->getChild()->setValue(
                    stoi(value)
                );
            }else if(param == "nEvalS"){
                n_eval_s->getChild()->setValue(
                    stoi(value)
                );
            }else if(param == "mutRate"){
                mut_rate->getChild()->setValue(
                    stod(value)
                );
            }else if(param == "crossRate"){
                cross_rate->getChild()->setValue(
                    stod(value)
                );
            }else if(param == "tournSize"){
                tourn_size->getChild()->setValue(
                    stoi(value)
                );
            }else if(param == "popSize"){
                pop_size->getChild()->setValue(
                    stoi(value)
                );
            }else if(param == "nbIter"){
                nb_iter->getChild()->setValue(
                    stoi(value)
                );
    		}

        }


    }else{
        cout << "Unable to find config.config file" << endl;
    }
}

void MasterTab::saveCfg(){

    ifstream config;
    ofstream config_out;
    config.open("../src/config.config");
    config_out.open("../src/config.tmp");

    if(config_out.is_open()){

        if(config.is_open()){

            string line;

            while (getline(config,line)){

                if(line.length() < 2) {
                    config_out << line << endl;
                    continue;
                };
                if(line.at(0) == '/' && line.at(1) == '/') {
                    config_out << line << endl;
                    continue;
                };
            
                size_t pos = line.find("=");
                string param = line.substr(0,pos);
                string value = line.substr(pos+1);

                config_out << param << "=";

                if(param == "romPath"){
                    config_out << "../ale/roms/" << rom_path->getChild()->currentText().toUtf8().constData();
    	    	}else if(param == "displayScreen"){
                    config_out << (display_screen->getChild()->isChecked()?"true":"false");
    	    	}else if(param == "enableSound"){
                    config_out << (enable_sound->getChild()->isChecked()?"true":"false");
                }else if(param == "repeatActionProb"){
                    config_out << repeat_action_prob->getChild()->value();
                }else if(param == "colourPalette"){
                    config_out << colour_palette->getChild()->currentText().toUtf8().constData();
                }else if(param == "freshPop"){
                    config_out << (fresh_pop->getChild()->isChecked()?"true":"false");
                }else if(param == "nEvalP"){
                    config_out << n_eval_p->getChild()->value();
                }else if(param == "nEvalS"){
                    config_out << n_eval_s->getChild()->value();
                }else if(param == "mutRate"){
                    config_out << mut_rate->getChild()->value();
                }else if(param == "crossRate"){
                    config_out << cross_rate->getChild()->value();
                }else if(param == "tournSize"){
                    config_out << tourn_size->getChild()->value();
                }else if(param == "popSize"){
                    config_out << pop_size->getChild()->value();
                }else if(param == "nbIter"){
                    config_out << nb_iter->getChild()->value();
                }else if(param == "algoGen"){
                    config_out << "true";
                }
                config_out << endl;

            }


            config.close();
            config_out.close();
            remove("../src/config.config");
            rename("../src/config.tmp", "../src/config.config");

        }else{
            cout << "Error opening the config file" << endl;
            config_out.close();
        }


        
    
    }else{
        cout << "Error creating the new config file" << endl;
    }

}

void MasterTab::launchMaster(){
    saveCfg();

    ConsoleTab *consoletab = new ConsoleTab();
    window->addTab(consoletab, "Console");

    MasterThread *mt = new MasterThread(consoletab);
    mt->start();
}