#include "pcgroupbox.h"

#include <QObject>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QPushButton>
#include <QList>

string remove_comments(string s){
    size_t pos;
    if((pos = s.find("//")) != string::npos)
        return s.substr(0,pos);
    else return s;
}

string remove_spaces(string s){
   s.erase(remove(s.begin(),s.end(), ' '),s.end());
   return s;
}

string chop_name(string &s){
    size_t pos;
    string result;
    if((pos = s.find(":")) != string::npos){
        result = s.substr(0,pos);
        s.erase(0,pos+1);
    }
    else {
        result = s;
        s.erase(0,s.size());
    }
    return result;
}

vector<string> parse_machines(string s){
    vector<string> result;

    size_t pos;
    while((pos = s.find(";"))!= string::npos){
        result.push_back(s.substr(0,pos));
        s.erase(0,pos+1);
    }
    if(s.length()>0){
        result.push_back(s);
    }

    return result;

}

PCGroupBox::PCGroupBox(string line)
{
    line = remove_comments(line);
    line = remove_spaces(line);
    setTitle(chop_name(line).data());
    vector<string> machines_names = parse_machines(line);

    size = machines_names.size();

    QVBoxLayout *vbox = new QVBoxLayout;

    for (size_t i = 0; i < size; i++){

        QCheckBox *cb = new QCheckBox;
        cb->setText(machines_names[i].data());

        vbox->addWidget(cb);
    }

    QHBoxLayout *buttons = new QHBoxLayout;
    QPushButton *all = new QPushButton("All");
    QPushButton *none = new QPushButton("None");

    QObject::connect(
        all,
        SIGNAL(clicked()),
        this,
        SLOT(checkAll())
    );

    QObject::connect(
        none,
        SIGNAL(clicked()),
        this,
        SLOT(uncheckAll())
    );

    buttons->addWidget(all);
    buttons->addWidget(none);

    vbox->addLayout(buttons);

    this->setLayout(vbox);
}


int PCGroupBox::is_empty(){
    return (size == 0);
}

vector<string> PCGroupBox::selected_machines(){
    vector<string> result;
    QList<QCheckBox *> cbs = this->findChildren<QCheckBox *>();
    int size = cbs.size();
    for(int i = 0; i<size; i++){
        if(cbs.at(i)->isChecked()){
            result.push_back(cbs.at(i)->text().toUtf8().constData());
        }
    }

    return result;
}

void PCGroupBox::print_selected_machines(){
    vector<string> selected = this->selected_machines();
    size_t size = selected.size();
    for(size_t i = 0; i< size; i++){
        cout << selected[i] << endl;
    }
}

void PCGroupBox::checkAll(){
    QList<QCheckBox *> cbs = this->findChildren<QCheckBox *>();
    int size = cbs.size();
    for(int i = 0; i<size; i++){
        cbs.at(i)->setChecked(true);
    }
}

void PCGroupBox::uncheckAll(){
    QList<QCheckBox *> cbs = this->findChildren<QCheckBox *>();
    int size = cbs.size();
    for(int i = 0; i<size; i++){
        cbs.at(i)->setChecked(false);
    }
}