/*
    This file is part of Leela Zero.
    Copyright (C) 2017 Gian-Carlo Pascutto

    Leela Zero is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Leela Zero is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Leela Zero.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "Utils.hpp"

/**
* @file Utils.cpp
* @brief Handle a set of useful method (ALE, config...)
*
*/

using namespace std;

Utils::ThreadPool thread_pool;

size_t Utils::ceilMultiple(size_t a, size_t b) {
    if (a % b == 0) {
        return a;
    }
    auto ret = a + (b - a % b);
    return ret;
}

int Utils::secam_to_range_index(int i)
{
  switch (i)
    {
    case 58:
      return 1;
    case 121:
      return 2;
    case 152:
      return 4;
    case 188:
      return 8;
    case 217:
      return 16;
    case 233:
      return 32;
    case 255:
      return 64;
    default:
      return 128;
    }
}

void Utils::secam210x160_to_42x32(const std::vector<unsigned char>& screen,
                                  std::vector<unsigned char>& screen42x32)
{
  int k = 0;
  for (int y = 0; y < 210; y += 5)
    {
      for (int x = 0; x < 160; x += 5)
        {
          unsigned char res = 0;
          for (int yy = y; yy < y + 5; ++yy)
            {
              for (int xx = 0; xx < x + 5; ++xx)
                {
                  res |= secam_to_range_index(screen[160 * yy + xx]);
                }
            }
          screen42x32[k++] = res;
        }
    }
}

vector<vector<string>> Utils::read_Config(){
	vector<vector<string>> config;
	ifstream file("config.config", ios::in); //Tentative d'ouverture du fichier
	if(file){
		string line;
		while(getline(file,line)){//Lecture du fichier ligne par ligne 
			if(line.length() != 0){//Pas de lecture de ligne vide
				if(!(line.at(0) == '/' && line.at(1) == '/')){//Pas de lecture des commentaires
					vector<string> words;
					int pos = 0;
					while((pos = line.find("=")) != -1){//split de la ligne lue selon un délimiteur, stockage dans un vector
						words.push_back(line.substr(0,pos));
						line.erase(0, pos+1);
					}
					words.push_back(line);//récupération du dernier mot de la ligne

					config.push_back(words);
				}
			}
		}
		file.close(); //Fermeture du fichier
	}
	return config;
}

ALEInterface ALE;

void Utils::init_ALE()
{	
	//Valeur par défaut
	float repeatActionProb = 0.25;
	bool displayScreen = false;
	bool enableSound = false;
	string romPath = "../ale/roms/Frostbite.bin";
	string colourPalette = "SECAM";

	//Lecture de la config
	vector<vector<string>> config = read_Config();

	for(size_t i = 0; i<config.size(); i++){
		if(config.at(i).at(0) == "romPath"){
			romPath = config.at(i).at(1);
		}else if(config.at(i).at(0) == "displayScreen"){
			if(config.at(i).at(1) == "true"){
				displayScreen = true;
			}else if(config.at(i).at(1) == "false"){
				displayScreen = false;
			}
		}else if(config.at(i).at(0) == "enableSound"){
			if(config.at(i).at(1) == "true"){
				enableSound = true;
			}else if(config.at(i).at(1) == "false"){
				enableSound = false;					
			}
		}else if(config.at(i).at(0) == "repeatActionProb"){
			repeatActionProb = stof(config.at(i).at(1));
		}else if(config.at(i).at(0) == "colourPalette"){
			colourPalette = config.at(i).at(1);
		}
	}	

	//Application de la config	
	ALE.setFloat("repeat_action_probability", repeatActionProb);
	ALE.setBool("display_screen", displayScreen);
	ALE.setBool("sound", enableSound);
	ALE.loadROM(romPath);
	ALE.theOSystem->colourPalette().setPalette("standard", colourPalette);
}
