#include "ga.hpp"
#include <random>

/**
* @file ga.cpp
* @brief Handle the genetic algorithm implementation
*
*/

ga::ga(const parameters& params, const agent::parameters& agent_params)
  : params(params),
    current_population(params.population_size, agent_params),
    new_population(params.population_size, agent_params),
    population_fitness(params.population_size)
{
  assert((params.population_size & 1) == 0);
}

void ga::init_population()
{
  for (int i = 0; i < params.population_size; ++i)
    {
      genetic_operators::initialize(current_population[i]);
    }
}

void ga::fill_population()
{
  std::vector<agent> current_population_retrieved;
  std::ifstream ifpop("best_pop.txt", std::ios::binary);
  boost::archive::text_iarchive ipop(ifpop);
  ipop >> current_population_retrieved;

  int sp = (int)current_population_retrieved.size();
  for(int i = 0; i < params.population_size; ++i)
  {
    if(i >= sp){
      genetic_operators::initialize(current_population[i]);
    }else{
      current_population[i] = current_population_retrieved[i];
    }
  }
}

int ga::selection() const
{
  int best_agent = 0;
  double best_value = std::numeric_limits<double>::lowest();
  std::uniform_int_distribution<int> r(0, params.population_size - 1);
  for (int i = 0; i < params.tournament_size; ++i)
    {
      int k = r(Utils::Random::get_Rng());
      if (population_fitness[k] > best_value)
        {
          best_agent = k;
          best_value = population_fitness[k];
        }
    }
  return best_agent;
}

void ga::select_population()
{
  for (int i = 0; i < params.population_size; ++i)
    {
      new_population[i] = current_population[selection()];
    }
}

void ga::crossover_population()
{
  std::uniform_real_distribution<float> r(0, 1);
  int sp = std::nearbyint(params.population_size * 0.5f) * 2.0f;
  for (int i = 0; i < sp; i += 2)
    {
      if (r(Utils::Random::get_Rng()) < params.crossover_rate)
        genetic_operators::crossover(new_population[i], new_population[i + 1]);
    }
}

void ga::mutate_population()
{
  std::uniform_real_distribution<float> r(0, 1);
  for (int i = 0; i < params.population_size; ++i)
    {
      if (r(Utils::Random::get_Rng()) < params.mutation_rate)
        genetic_operators::mutate(new_population[i]);
    }
}
