﻿#include "agent.hpp"
#include "agent_worker.hpp"
#include "master_slave.hpp"
#include <fstream>
#include <boost/archive/text_iarchive.hpp>
#include "Utils.hpp"

/**
* @file main.cpp
* @brief Start the algorithm, either in exhibition or training mode.
*
*/

/**
 * @mainpage IAtari Documentation
 *
 * @section install How to install IAtari ?
 *
 * Retrieve the git repository (git clone for instance).\n
 * Make sure to have a compiler that support c++ 14, the most recent version of g++ should do the trick.\n
 * You need to install the boost library : "apt-get install libboost-all-dev".\n
 * Normally, you don't have to install ALE. Nethertheless, you need to set the environment variable LD_LIBRARY_PATH. To do so, type "export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:/my/path/to/ale/directory". It is extremely recommended to put this command in your .bashrc so that the env variable will be set permanently.\n
 * Type "make" in the "src" directory to compile the algorithm. "make clean" will remove .o, .d and executable file.
 *
 * @section run How to launch or train your AI ?
 * @subsection launch Launch (Exhibition mode) :
 *
 * In the config file, set the parameter "algogen" to false, you can also play with the "ALE settings".\n
 * Once the program is compiled, execute the command "./atari"
 *
 * @subsection train Train (Algo-gen mode) :
 *
 * In the config file, set the parameter "algogen" to true and "colourPalette" to SECAM, you can also play with the "ALE settings". Nethertheless, it is highly recommended to set "displayScreen" and "enableSound" to false, otherwise it will considerably slow down the tests.\n
 * Once the program is compiled, execute the command "./atari -m" in a terminal to start the master.\n
 * Then, you can launch as many slave as you want by executing "./atari -s" in others terminals. To launch the algorithm in the entire department use the script named "launch_depart.sh" or the GUI.
 *
 * @section algo How does the algorithm work ?
 * @subsection algofl Algorithm flow
 * @image html ../algo_flow.png
 */

int main(int argc, char *argv[])
{
  using namespace std;
  Utils::init_ALE();

  //Valeur par défaut
  bool algoGen = false;

  //Lecture de la config
  vector<vector<string>> config = Utils::read_Config();
  
  for(size_t i = 0; i<config.size(); i++){
    if(config.at(i).at(0) == "algoGen"){
      if(config.at(i).at(1) == "true"){
        algoGen = true;
      }else if(config.at(i).at(1) == "false"){
        algoGen = false;
      }
    }
  } 

  //Application de la config
  if(algoGen){//Algo génétique
    master_slave::run(argc, argv); 
  }else{//Tester un agent particulier
    agent a;
    std::ifstream ifs("best_agent.txt", std::ios::binary);
    boost::archive::text_iarchive archive(ifs);
    archive >> a;

    cout << a << endl;
    
    message::request<agent> request;

    for (int i = 1; i < 10; ++i)
    {
      request.nb_eval = 1;
      request.seed = i;
      request.params = a;
      message::result<double> res = agent_worker(request);
    }
  }

  return 0;
}

// dans une première fenêtre ./atari -m
// et dans une autre, soit lancer launch.sh, ou ./atari -s si on veut rester en local

